﻿var moment = require('moment-timezone');
var Datastore = require('nedb');

//this file used to create access tockens that can be used for once, 
var Tokens = function (app) {
    this.init(app, function () { });
}

Tokens.prototype = {
    init: function (app) {
        var instance = this;
        instance.app = app;
        instance.machiModule = app.get('machiModule');
        instance.options = instance.machiModule.options;
    },
    _openDB: function (callback) {
        var db = new Datastore({ filename: 'tokens.db', autoload: true });
        db.loadDatabase(function () { callback(db); });
    },
    /** 
    * Create new access tocken for this req
    *
    * @param {string} key: the key used to generate the token
    * @param {object} value: the value object
    * @param {Number} expireTime: the time in seconds
    */
    createToken: function (key, value, expireTime, length) {
        var instance = this;
        var tocken = instance.generateToken(length || 50);
        instance._openDB(function (db) {
            db.insert({
                key:key,
                token: tocken,
                value: value,
                expire: moment().add(expireTime, 's').format('YYYYMMDDHHmmss')
            });
        });
        return tocken;
    },
    /** 
   * Get the value that generated this access tocken
   *
   * @param {string} key: the key used to generate the token
   * @param {String} token: the access tocken
   * @param {boolean} removeToken: should remove the tocken?
   * @param {function} callback: the callback that will hold the value
                                  -err: in case any error
                                  -req: the request object that used to generate the access tocken
   */
    getToken: function (key, token, removeToken, callback) {
        var instance = this;
        instance._openDB(function (db) {
            db.find({key:key, token: token }, function (err, docs) {
                if (err || !docs) return callback('Token not valid');
                var validDocs = docs.filter(function (d) {
                    return moment(d.expire, 'YYYYMMDDHHmmss').diff(moment()) > 0;
                })[0];
                if (removeToken)
                    db.remove({ key: key, token: token }, { multi: true });
                if (!validDocs) return callback('Token not valid');
                callback(undefined, validDocs.value);
            });
        });
    },
    removeToken: function (key, token) {
        var instance = this;
        instance._openDB(function (db) {
            db.remove({ key: key, token: token }, { multi: true });
        });
    },
    generateToken: function (length) {
        var text = "";
        var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

        for (var i = 0; i < length; i++)
            text += possible.charAt(Math.floor(Math.random() * possible.length));

        return text;
    }
}

exports.Tokens = Tokens;