var MachiConnect = require('./lib/connect.js').MachiConnect;
var MachiEmails = require('./lib/machi-emails.js').MachiEmails;
var Tokens = require('./lib/tokens.js').Tokens;
var TwoFactorAuth = require('./lib/two-factor-auth.js').TwoFactorAuth;
var Sendgrid = require('./lib/sendgrid-emails.js').Sendgrid;
var MachiUtilities = require('./lib/utility').MachiUtilities;

var express = require('express');
var ejs = require('ejs');

/**
 * Create an instance of machi-auth
 * @method 
 * @param app: Express App
 * @returns {{The express middleware}}
 */
var MachiAuth = function () {

}

module.exports.MachiAuth = MachiAuth;
module.exports.MachiConnect = MachiConnect;
module.exports.MachiEmails = MachiEmails;
module.exports.Sendgrid = Sendgrid;


MachiAuth.prototype = {
    /**
     * Initialize the machi-auth package 
     * @method 
     * @param app: Express App
     * @returns {{The express middleware}}
     */
    init: function (app, options) {
        if (!app) throw 'Options is missing the express App';
        if (!options.baseURL) throw console.error( 'Options is missing the base URL');
        if (!options.membershipDB) throw console.error('Options is missing the membership database string connection (membershipDB)');
       // if (!options.apiKey) throw console.error('Options is missing the apiKey');
        //options.oktaDomain = options.oktaDomain || 'machi';
        //options.oktapreview = options.oktapreview;
        if (!options.sendgridApiKey) throw console.error('Options is missing the sendgridApiKey');
        options.jwtSecret = options.jwtSecret || 'MyTokenSecret2018';
        app.useIfNotExists = useIfNotExists;

        app.useIfNotExists(require('compression')());
        app.useIfNotExists(require('cookie-parser')());
        app.useIfNotExists(require('request-param')());
        app.useIfNotExists(require('body-parser').json());
        app.useIfNotExists(require('body-parser').urlencoded({ extended: true }));

        app.set('view engine', 'html');
        app.engine('html', ejs.renderFile);
        app.use('/machiAuthStatic', express.static(__dirname + '/lib/css'));

        var instance = this;
        instance.app = app;
        instance.options = options;
        app.set('machiModule', instance);
		
        instance.sendgrid = new Sendgrid(options.sendgridApiKey);
        instance.tokens = new Tokens(app);
        instance.twoFactorAuth = new TwoFactorAuth();
		instance.emails = new MachiEmails(app);
        instance.connect = new MachiConnect(app);
        instance.utilities = new MachiUtilities(app);
        instance.middleware = require('./lib/middleware.js')(app);
        instance.cookieMiddleware = instance.middleware.cookie = require('./lib/cookies')(options.sessionOptions);
        instance.middleware.auth = require('./lib/passport-express.js')(app);

       
        return instance;
    },
    /**
     * Defines the machi-auth express routes 
     * @method 
     * @param app: Express App
     */
    defineRoutes: function () {
        var instance = this,
            app = instance.app;
        app.use(instance.middleware.allowOriginHeaders);
        require('./lib/routes/web.js')(app);
        require('./lib/routes/user-api.js')(app);
    }
}

function middlewareExists(app, name) {
    return !!app._router.stack.filter(function (layer) {
        return layer && layer.handle && layer.handle.name === name;
    }).length;
}

function useIfNotExists(router) {
    var app = this;
    if (!router.name || !middlewareExists(app, router.name))
        app.use(router);
}