﻿var pg = require('./pg/connect.js');
var md5 = require('md5');

function hashPassword(password) {
    return md5('@#A%_' + password + '_12&*');
}
var insert_user_query = `insert into membership.user_account (email,username,password_hash,phone_number,status) values ($1,$2,$3,$4,'staged') RETURNING id`;
var insert_profile_query = `insert into membership.user_profile (user_id,display_name,picture,gender,location,website,organization,title,best_time_to_call,is_public,user_data) values ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11)`;

var select_user_query = `
                        select 
                              u.id,u.email,u.username,u.phone_number,u.phone_number_confirmed,u.email_confirmed,u.two_factor_enabled,u.two_factor_secret,u.password_hash,u.status,
                              p.display_name,p.picture,p.gender,p.location,p.website,p.created_at,p.updated_at,p.organization,p.title,p.best_time_to_call,p.is_public,p.user_data
	  
                        from membership.user_account u
                        left outer join membership.user_profile p on u.id=p.user_id

                        where u.email=$1 or u.username=$2`;


var update_user_query = `
                       update 
                             membership.user_account 
                       set 
                             email=$2,email_confirmed=$3,phone_number=$4,phone_number_confirmed=$5,two_factor_enabled=$6,two_factor_secret=$7,status=$8
                       where 
                             id=$1;`;
var update_profile_query = `
                       update
                             membership.user_profile
                       set
                             display_name=$2,picture=$3,gender=$4,location=$5,website=$6,organization=$7,title=$8,best_time_to_call=$9,is_public=$10,user_data=$11
                       where
                       user_id=$1`;
var update_password_query = `
                       update 
                             membership.user_account 
                       set 
                             password_hash=$2
                       where 
                             id=$1;`;
var update_password_status_query = `
                       update 
                             membership.user_account 
                       set 
                             password_hash=$2,status='active'
                       where 
                             id=$1;`;
var select_user_groups_query = `
                        select 
                              g.name
                        from  membership.groups g
                        join  membership.group_users gu 
                        on    g.id=gu.group_id
                        join  membership.user_account u
                        on    gu.user_id=u.id
                        where u.username=$1`;
var add_user_to_group_query = `
                        insert into 
                               membership.group_users
                               (user_id,group_id) values ($1,$2)`;
var MachiConnect  = function (app) {
    this.init(app);
}

MachiConnect.prototype = {
    /**
     * Initlize the Connector instance
     *
     * @method
     *
     * @param {Express} app - The express app
     */
    init: function (app) {
        var instance = this;
        this.app = app;
        var machiModule = instance.machiModule = app.get('machiModule');
        var options = machiModule.options;
        instance.db = pg(options.membershipDB);
        instance.emails = machiModule.emails;
        instance.options = options;

        //The token link expire in 24 hours
        instance.forgotPasswordToken_Expire = options.forgotPasswordToken_Expire || 60 * 60 * 24;
        //The token link expire in 7 days
        instance.activationToken_Expire = options.activationToken_Expire || 60 * 60 * 24 * 7;
    },
    /**
     * Creates a new user that reflects the structure of an User Object
     * @method addUser
     * @param email
     * @param username
     * @param phone_number optional
     * @param user_data optional- a map of property -> propertyValue for custom attributes on a user
     * @returns {{}}
     */
    addUser: function (profile, callback) {
        var instance = this;
        instance.getUser(profile.username, profile.email, function (err, user) {
            if (!err && user)
                return callback('Username ' + user.username + ' already exists. please use differnt email address');
            instance._checkAll(profile, (err) => {
                if (err) return callback(err);
                var args_user = [profile.email, profile.username, hashPassword(profile.password), profile.phone_number];

                instance.db.open((err, client, done) => {
                    if (err) return callback(err);

                    client.query(insert_user_query, args_user, (err, result) => {
                        if (err) { done(err); return callback(err); }
                        //insert the other user profile                    
                        var args_profile = [result.rows[0].id, profile.display_name, profile.picture, profile.gender, profile.location, profile.website, profile.organization, profile.title, profile.best_time_to_call, profile.is_public, profile.user_data];
                        client.query(insert_profile_query, args_profile, (err, result) => {
                            done(err);
                            if (err) return callback(err);
                            instance.sendActivationEmail(profile.username, function(err) {
                                if (err) return callback('Field to send activation link, please contact support team');
                                //return the full-user profile from database
                                instance.getUser(profile.username, callback);
                            });

                        });
                    });

                });
                
            });

        });
        
    },
    /**
     * Check if user email is valid and password is satisfying the minimum conditions
     * @method getUser
     * @returns {{}}
     */
    checkUserProfile: function(profile, callback) {
        var instance = this;

        if (!profile.email)
            return callback('Please provide an email address for your profile');
        if (!profile.username)
            return callback('Please provide a username for your profile');
        if (!validateEmail(profile.email))
            return callback('Please provide a valid email address for your profile');
        callback();
    },
    checkUserPassword: function(password, callback) {
        
        if (!password || password.length < 8)
            return callback('Please provide a valid password for your profile, Password must be more than 8 letters.');
        callback();
    },
    _checkAll: function(profile, callback) {
        var instance = this;
        instance.checkUserProfile(profile, err => {
            if (err) return callback(err);
            instance.checkUserPassword(profile.password, callback);
        });
    },
    /**
     * Search for a user by it's username or by it's email
     * @method getUser
     * @returns {{}}
     */
    getUser: function(username, email, callback) {
        var instance = this;
        if (typeof (email) == 'function') {
            callback = email;
            email = username;
        }

        instance.db.query(select_user_query, [email, username], (err, result) => {
            if (err) return callback(err);
            if (result.rows.length == 0)
                return callback('User not exists');
            callback(undefined, result.rows[0]);
        });
        
    },
    /**
     * Sends an email that has a link to activate this User
     * @method sendActivationEmail
     * @param username
     * @returns {{}}
     */
    sendActivationEmail: function (username, callback) {
        var instance = this;
        var tokens = instance.machiModule.tokens;
        tokens.createToken(TokenKeys.User, username, instance.activationToken_Expire, (err, token) => {
            if (err) return callback(err);
            var activationUrl = instance.options.baseURL + instance.options.routes.activateUser + '?token=' + token;
            instance.emails.sendEmail(username, 'verify', {
                activationUrl: activationUrl
            }, callback);
        });
       
    },

    /**
     * Updat user's profile
     * @method updateUser
     * @param username
     * @param profile
     * @returns {{}}
     */
    updateUser: function(username, profile, callback) {
        var instance = this;
        instance.getUser(username, (err, user) => {
            if (err) return callback(err);
            //make sure the update profile has the new values
            Object.keys(profile).forEach(key => { user[key] = profile[key] });
            instance.checkUserProfile(user, err => {
                if (err) return callback(err);

                var args1 = [user.id, user.email, user.email_confirmed, user.phone_number, user.phone_number_confirmed, user.two_factor_enabled, user.two_factor_secret, user.status];
                var args2 = [user.id, user.display_name, user.picture, user.gender, user.location, user.website, profile.organization, profile.title, profile.best_time_to_call, profile.is_public, user.user_data];

                instance.db.open((err, client, done) => {
                    if (err) return callback(err);

                    client.query(update_user_query, args1, (err) => {
                        if (err) { done(err); return callback(err); }

                        client.query(update_profile_query, args2, (err) => {
                            done(err);
                            callback(err, !err && user);
                        });

                    });

                });

            });
            });
    },
    /**
     * Updat user's profile and password
     * @method changeUserPassword
     * @param username
     * @param password
     * @param sendEmail
     */
    changeUserPassword: function(username, password, sendEmail, callback) {
        if (typeof (sendEmail) == 'function') {
            callback = sendEmail;
            sendEmail = true;
        }
        var instance = this;
        instance.checkUserPassword(password, err => {
            if (err) return callback(err);

            instance.getUser(username, (err, user) => {
                if (err) return callback(err);
                var args = [user.id, hashPassword(password)];
                var query = user.status == 'password reset' ? update_password_status_query : update_password_query;
                instance.db.query(query, args, err => {
                    if (err || !sendEmail) return callback(err);
                    instance.emails.sendEmail(username, 'passwordChanged', {}, callback);
                });
            });

        });
      
        
    },
    /**
     * Activate a new user 
     * @method activateUser
     * @param username: The user id
     * @param sendEmail: should send an email
     * @param callback
     * @returns {{}}
     */
    activateUser: function (username, sendEmail, callback) {
        var instance = this;
        if (typeof (sendEmail) == 'function') {
            callback = sendEmail;
            sendEmail = true;
        }
        instance.updateUser(username, { email_confirmed: true, status:'active' }, function(err, user) {
            if (err) return callback(err);
            if (sendEmail)
                instance.emails.sendEmail(username, 'welcome', {}, callback);
            else callback(undefined, user);
        });

    },


    /**
     * Reset User's password link
     * @method createForgotPasswordLink
     * @param username: The user id
     * @param callback
     * @returns {{}}
     */
    createForgotPasswordLink: function (username,callback) {
        var instance = this;
        var tokens = instance.machiModule.tokens;
        tokens.createToken(TokenKeys.Password, username, instance.forgotPasswordToken_Expire, (err, token) => {
            if (err) return callback(err);
            var url = instance.options.baseURL + instance.options.routes.resetPassword + '?token=' + token;
            callback(undefined, url);
        });
    },


    /**
     * Reset User's password link
     * @method sendForgotPasswordEmail
     * @param username: The user id
     * @param callback
     * @returns {{}}
     */
    sendForgotPasswordEmail: function(username, callback) {
        var instance = this;
        instance.createForgotPasswordLink(username, (err, url) => {
            if (err) return callback(err);
            instance.emails.sendEmail(username, 'resetPassword', {
                resetPasswordURL: url
            }, callback);
        });
    },
    


    /**
     * Reset User's password link
     * @method createForgotPasswordLink
     * @param username: The user id
     * @param callback
     * @returns {{}}
     */
    createConfirmEmailLink: function (username,newEmail,callback) {
        var instance = this;
        var tokens = instance.machiModule.tokens;
        tokens.createToken(TokenKeys.ConfirmEmail, { username: username, newEmail: newEmail }, (err, token) => {
            if (err) return callback(err);
            var url = instance.options.baseURL + instance.options.routes.confirmEmail + '?token=' + token;
            callback(undefined, url);
        });
    },


    /**
     * Confirm User's email link
     * @method sendForgotPasswordEmail
     * @param username: The user id
     * @param callback
     * @returns {{}}
     */
    sendConfirmEmailEmail: function(username, newEmail, callback) {
        var instance = this;
        instance.createConfirmEmailLink(username, newEmail, (err, url) => {
            if (err) return callback(err);

            instance.emails.sendEmailTo(username, newEmail, 'confirmEmail', {
                confirmEmailUrl: url
            }, callback);

        });
    },
    /**
     * Gets all groups assigned to this user
     * @method getUserGroups
     * @param username: The user id
     * @param callback
     * @returns {{}}
     */
    getUserGroups: function (username, callback) {
        var instance = this;
        instance.db.query(select_user_groups_query, [username], (err, result) => {
            if (err) return callback(err);
            callback(undefined, result.rows);
        });

    },
    /**
     * Add this user to this group
     * @method addUserToGroup
     */
    addUserToGroup: function (user_id,group_id, callback) {
        var instance = this;
        instance.db.query(add_user_to_group_query, [user_id, group_id], (err, result) => {
            if (err) return callback(err);
            callback(undefined, result.rows);
        });

    },
    /**
     * check if this user/pass are valid
     *
     */
    validateLogin: function (username, password, callback) {
        var instance = this;
        instance.getUser(username, (err, user) => {
            if (err) return callback(err);
            if (user.password_hash != hashPassword(password))
                return callback('Username and Password not match');
            if (user.status == 'disabled')
                return callback('Your account is disabled.');
            if (!user.email_confirmed || user.status == 'staged')
                return callback('Please confirm your email address before login.', user);
            if (user.status == 'password reset')
                return callback('Your password must be reset before login.', user);
            
            callback(undefined,user);
        });
        

    },
    //reply with bad auth header user/pass
    logout: function (req, res, redirectToIndex) {
        var instance = this;

        if (req.session && req.session.destroy)
            req.session.destroy();
        if (req.cookies) {
            Object.keys(req.cookies).forEach(function (name) {
                res.clearCookie(name);
            });
        }
        req.user = undefined;
        if (redirectToIndex)
            res.redirect('/');
    }



}


function validateEmail(email) {
    var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(String(email).toLowerCase());
}
module.exports.MachiConnect = MachiConnect;