﻿var pg = require('pg');
var PgClient = pg.Client;

/**
 * Create instance of connect DB
 *
 * @method
 *
 * @param {function} callback - The function that will handle the opened connection
 */
module.exports = function(connectionString) {
    var connect = {};

    /**
     * Start a connection to the database
     *
     * @method
     *
     * @param {function} callback - The function that will handle the opened connection
     */
    connect.open = function(callback) {

        var client = new PgClient({ connectionString: connectionString });
        client.connect(err => {
            if (err) return callback(err);

            client.query('BEGIN', err => {
                if (err) return callback(err);

                callback(undefined, client, (err) => {

                    var command = err ? 'ROLLBACK' : 'COMMIT';
                    client.query(command, () => client.end());

                });
            });
        });

    }


    /**
     * Execute a query
     *
     * @method
     *
     * @param {string} query - The query text
     * @param {array} paramsArray - The query args
     * @param {function} callback - The function that will handle the query result
     */
    connect.query = function(query, paramsArray, callback) {

        var client = new PgClient({ connectionString: connectionString });
        client.connect(err => {
            if (err) return callback(err);
            client.query(query, paramsArray, function(err, result) {
                client.end();
                callback(err, result);
            });
        });

    }

    return connect;
}
