﻿var express = require('express');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var oktaAuth = require('okta-auth');
var session = require('express-session');
var redis = require('redis');
var okta = require('./connect.js');
var RedisStore = require('connect-redis')(session);
var fs = require('fs');
var app = global.expressApp= express();



app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());

var sessionOptions = {
    //Keeps login for one month
    cookie: { maxAge: 30 * 24 * 60 * 60 * 1000 },
    name: 'Machina',
    resave: false,
    secret: process.env.SESSION_SECRET,
    saveUninitialized: false,
   // rolling: true
};


if (process.env.REDIS_URL) {
    const redisClient = redis.createClient({ url: process.env.REDIS_URL });
    sessionOptions.store = new RedisStore({
        client: redisClient,
        ttl: sessionOptions.cookie.maxAge
    });
} else { // Use file store on local environment
    try {
        var FileStore = require('session-file-store')(session);
        sessionOptions.store = new FileStore({
            path: '/tmp/sessions',
            ttl: sessionOptions.cookie.maxAge
        });
    } catch (e) {
        console.error(e);
    }
}


app.use(session(sessionOptions));

oktaAuth.initialize(app, {
    oktaIssuer: process.env.OKTA_ISSUER,          // required
    oktaEntryPoint: process.env.OKTA_ENTRY_POINT, // required
    oktaCert: fs.readFileSync('./okta.cert', 'utf8'),             // required
    logoutUrl:'/something we don\'t want'
});

app.get('/logout', (req, res) => {
    req.session.destroy();
   // res.redirect('/');
    okta.logout(req, res, true);
});

module.exports = {
    app: app,
    secured: oktaAuth.secured
};


