﻿var moment = require('moment-timezone');

//this file used to create access tockens that can be used for once, 
var Tokens = function (app) {
    this.init(app, function () { });
}

Tokens.prototype = {
    init: function (app) {
        var instance = this;
        instance.app = app;
        instance.machiModule = app.get('machiModule');
        instance.options = instance.machiModule.options;
    },
    /** 
    * Create new access token for this req
    *
    * @param {string} key: the key used to generate the token
    * @param {object} value: the value object
    * @param {Number} expireTime: the time in seconds
    */
    createToken: function(key, value, expireTime, length,callback) {
        if (typeof (expireTime) == 'function') {
            callback = expireTime;
            expireTime = length = undefined;
        }
        if (typeof (length) == 'function') {
            callback = length;
            length = undefined;
        }

        var instance = this,
            db = instance.machiModule.connect.db;
        var token = instance.generateToken(length || 50);
        expireTime = expireTime || (60 * 60 * 24);
        var sql = `insert into membership.tokens (token,key,value,expire_at,value_type) values ($1,$2,$3,$4,$5)`;
        var args = [token, key, value, moment().add(expireTime, 's').format('YYYY-MM-DD HH:mm:ss'), typeof (value)];
        db.query(sql, args, err => {
            if (err) return callback(err);
            callback(undefined, token);
        });
    },
    /** 
   * Get the value that generated this access token
   *
   * @param {string} key: the key used to generate the token
   * @param {String} token: the access token
   * @param {boolean} removeToken: should remove the token?
   * @param {function} callback: the callback that will hold the value
                                  -err: in case any error
                                  -req: the request object that used to generate the access token
   */
    getToken: function(key, token, removeToken, callback) {
        var instance = this,
            db = instance.machiModule.connect.db;
        var sql = `select * from membership.tokens where token=$1 and key=$2 and expire_at > CURRENT_TIMESTAMP and is_enabled=true `;
        var args = [token, key];
        db.query(sql, args, (err, result) => {
            if (err) return callback(err);
            if (!result.rows.length)
                return callback('Token not valid');
            var value_type = result.rows[0].value_type;
            var value = result.rows[0].value;
            if (value_type == 'object')
                try { value = JSON.parse(value); }
                catch (err) { console.error(err); }

            if (!removeToken)
                return callback(undefined, value);

            instance.removeToken(key, token, err => {
                callback(undefined, value);
            });

        });

    },
    removeToken: function (key, token,callback) {
        var instance = this,
            db = instance.machiModule.connect.db;
        var deleteSql = 'delete from membership.tokens where token=$1 and key=$2 ';
        db.query(deleteSql, [token, key], callback);
    },
    generateToken: function (length) {
        var text = "";
        var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

        for (var i = 0; i < length; i++)
            text += possible.charAt(Math.floor(Math.random() * possible.length));

        return text;
    }
}

global.TokenKeys = {};
TokenKeys.ConfirmEmail = 'ConfirmEmail';
TokenKeys.User = 'User';
TokenKeys.Password = 'Password';
exports.Tokens = Tokens;